# frozen_string_literal: true

module RailsRbs
  # This module of helpers is utilized by the RailsRbs generators to detect model and migration files
  # at their correct path
  module MigrationHelpers

    # Detect if the rules and rule_sets table migrations exist.
    # @return true if the rules and rule_sets migration files exist
    def rule_migrations_exist?
      %w[add_rules add_rule_sets add_rule_actions].any? do |name|
        Dir.glob("#{File.join(destination_root, migration_path)}/[0-9]*_*.rb").grep(/\d+_#{name}.rb$/).first
      end
    end

    # Detect if the migrations for the default rule types exist.
    # @return true if date_range and location rule migration files exist
    def default_rule_migrations_exist?
      %w[add_date_range add_location].all? do |name|
        Dir.glob("#{File.join(destination_root, migration_path)}/[0-9]*_*.rb").grep(/\d+_#{name}_rule.rb$/).first
      end
    end

    # Detect if a specific model exists by singular name
    # @return true if the a file is found in app/models that matches the name provided
    def model_exists?(name: )
      File.exists?(File.join(destination_root, model_path(name: name)))
    end

    # Return the file path for a model using the name as the singular file name
    # @return [String] The relative path of the model file (app/models/rule.rb for name: rule)
    def model_path(name: )
      File.join("app", "models", "#{name}.rb")
    end

    # @return [String] The relative path to the directory where migration files are stored
    def migration_path
      if self.respond_to?(:db_migrate_path) # Rails 5.0.3 and up
        db_migrate_path
      else
        File.join("db", "migrate")
      end
    end
  end
end
