# frozen_string_literal: true

require 'rails/generators/active_record'
require 'generators/rails_rbs/migration_helpers'
require 'active_support/all'

module RailsRbs
  module Generators
    class RuleGenerator < Rails::Generators::NamedBase
      include ActiveRecord::Generators::Migration
      include RailsRbs::MigrationHelpers

      namespace 'rails_rbs:rule'

      source_root(File.expand_path('../templates', __FILE__))

      def generate_rule_model
        invoke("active_record:model", [model_name], migration: false, parent: 'Rule') unless model_exists?(name: model_name)
      end

      def generate_migrations
        unless default_rule_migrations_exist?
          if name == 'date_range'
            migration_template 'date_range_rule_migration.rb', "#{migration_path}/add_date_range_rule.rb"
          elsif name == 'location'
            migration_template 'location_rule_migration.rb', "#{migration_path}/add_location_rule.rb"
          end
        end
      end

      def inject_rule
        default_rules = %w[equality date_range location] # The built in rules of the gem
        if default_rules.include?(name)
          content = default_rule_content
        else
          content = empty_rule_content
        end
        inject_into_class(model_path(name: model_name), "#{name.to_s.camelize}Rule", content)
      end

      private
      def empty_rule_content
        <<-CONTENT
  def follows_rule?(*objects)
    # Your rule logic here!
    false
  end
        CONTENT
      end

      def default_rule_content
        <<-CONTENT
  default_rule :#{name}
        CONTENT
      end

      def model_name
        "#{name}_rule"
      end
    end
  end
end
