# frozen_string_literal: true

require 'rails'

module RailsRbs
  autoload :FollowsRule, 'rails_rbs/follows_rule'
  autoload :FollowsRuleSet, 'rails_rbs/follows_rule_set'
  autoload :RunsAction, 'rails_rbs/runs_action'

  module Rules
    autoload :Equality, 'rails_rbs/rules/equality'
    autoload :DateRange, 'rails_rbs/rules/date_range'
    autoload :Location, 'rails_rbs/rules/location'
  end

  module Controllers
    autoload :Helpers, 'rails_rbs/controllers/helpers'
  end

  module Models
    autoload :FollowsScope, 'rails_rbs/models/follows_scope'
  end

  # Default way to set up RailsRbs. Run rails generate rails_rbs to create
  # a fresh initializer with all configuration values.
  def self.setup
    yield self
  end

  mattr_accessor :owned_by
  @@owned_by = :user

  mattr_accessor :owner_access
  @@owner_access = :current_user

  mattr_accessor :render_views
  @@render_views = true

  mattr_accessor :request_id_field
  @@request_id_field = :id

  mattr_accessor :base_rule_fields
  @@base_rule_fields = %i[type enforced_type enforced_value observed_field rule_set_id comparison_operator]

  mattr_accessor :rule_fields
  @@rule_fields = %i[value start_date end_date longitude latitude]

  mattr_accessor :parent_controller
  @@parent_controller = 'ApplicationController'

  mattr_accessor :rule_params_root
  @@rule_params_root = :rule

  mattr_accessor :rule_set_params_root
  @@rule_set_params_root = :rule_set

  mattr_accessor :rule_set_view_dir
  @@rule_set_view_dir = 'rule_sets'

  mattr_accessor :rule_view_dir
  @@rule_view_dir = 'rules'
end

require 'rails_rbs/engine'
