module RailsRbs
  module RunsAction
    extend ActiveSupport::Concern

    included do
      belongs_to :rule_set
      validates_presence_of :enforced_field
      validates_presence_of :enforced_value
    end

    # Apply the action to all objects in the provided collection.  Application simply means
    # the enforced_field will be set to the enforced_value.
    # @param objects [Array<ActiveRecord::BAse> | ActiveRecord::Base] a single collection of active
    # record objects to have this rule_action applied to.
    def apply(*objects)
      objects.flatten.map do |object|
        if object.persisted?
          response = object.update_column(self.enforced_field.to_sym, self.enforced_value)
        else
          response = object.send("#{self.enforced_field.to_sym}=", self.enforced_value)
        end
        [response, object.try(:errors)]
      end
    end
  end
end
